/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.api.graph;

import java.util.Map;

import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.BladeFilter;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.supervisor.api.ClifException;

/**
 * The Interface DataSupplier.
 * @author Jordan BRUNIER
 * @author Gregory CALONNIER
 * @author Olivier LIU
 */
public interface DataSupplier {

    //  Fractal interface name
    /** The Constant DATA_SUPPLIER. */
    static public final String DATA_SUPPLIER = "Data supplier";
    
    /** The Constant STAT_STD_DEVIATION. */
    static public final String STAT_STD_DEVIATION = "Standard deviation";
    
    /** The Constant STAT_MEANS. */
    static public final String STAT_MEANS = "Means";
    
    /** The Constant STAT_MINIMUMS. */
    static public final String STAT_MINIMUMS = "Minimums";
    
    /** The Constant STAT_MAXIMUMS. */
    static public final String STAT_MAXIMUMS = "Maximums";
     

    /**
     * Get the blade descriptors
     * @param testName the test name 
     * @param filter the blade filter, null means no filter
     * @return an array that contains the blade descriptors
     */
    public BladeDescriptor[] getTestPlan(String testName,BladeFilter filter);
    
    /**
     * Get the tests' names.
     * 
     * @return an array that contains the tests' names
     */
    public String[] getTestsNames();
    
    
    /**
     * Get the baldes' ids.
     * 
     * @param testName the test run name
     * 
     * @return an array that contains the blades ids of the given test
     */
    public String[] getBladesIds(String testName);
    
    
    /**
     * Get the events labels.
     * 
     * @param testName the test run name
     * @param bladeId the blade identifier
     * 
     * @return an array that contains the events label of the given blade
     */
    public String[] getEventsTypeLabels(String testName, String bladeId);
    
    
    /**
     * Get the events fields labels.
     * 
     * @param testName the test run name
     * @param bladeId the blade identifier from the given test run
     * @param eventTypeLabel the label of the event type
     * 
     * @return an array that contains the labels of the given event type
     */
    public String[] getEventFieldLabels(String testName, String bladeId, String eventTypeLabel);
    
    
    /**
     * Get the values for the X and Y fields.
     * 
     * @param testName the test run name
     * @param bladeId the blade identifier from the given test run
     * @param eventTypeLabel the label of the event type
     * @param XFieldLabel the label of the X field
     * @param YFieldLabel the label of the Y field
     * @param start the start
     * @param count the count
     * @param end the end
     * 
     * @return a two dimension array that contains the values of the given
     *         fields
     */
    public Object[][] getFieldsValues(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count);
    
    
    /**
     * Get statistic on the Y field.
     * 
     * @param testName the test run name
     * @param bladeId the blade identifier from the given test run
     * @param eventTypeLabel the label of the event type
     * @param XFieldLabel the label of the X field
     * @param YFieldLabel the label of the Y field
     * @param start the start
     * @param end the end
     * @param count the count
     * 
     * @return a map that contains the various statistics on the Y field
     */
    public Map getStatistics(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count);
    
    
    /**
     * Count values for the giving field label.
     * 
     * @param testName the test run name
     * @param bladeId the blade id
     * @param eventTypeLabel the event type label
     * @param YFieldLabel the Y field label
     * 
     * @return the number of values
     */
    public long countValues(String testName, String bladeId, String eventTypeLabel, String YFieldLabel);

    
    /**
     * Gets the means.
     * 
     * @param values the values
     * @param timeWindow the window
     * @param step the step
     * 
     * @return the means
     */
    public Object[][] getMeans(Object[][] values, int timeWindow, int step);
    
    
    /**
     * Gets the standard deviation.
     * 
     * @param values the values
     * @param timeWindow the window
     * @param step the step
     * 
     * @return the standard deviation
     */
    public Object[][] getStandardDeviations(Object[][] values, int timeWindow, int step);
    
    
    /**
     * Gets the min values.
     * 
     * @param values the values
     * @param timeWindow the time window
     * @param step the step
     * 
     * @return the min values
     */
    public Object[][] getMinValues(Object[][] values, int timeWindow, int step);
    
    
    /**
     * Gets the max values.
     * 
     * @param values the values
     * @param timeWindow the time window
     * @param step the step
     * 
     * @return the max values
     */
    public Object[][] getMaxValues(Object[][] values, int timeWindow, int step);
    
    
    /**
     * Gets the max time for.
     * 
     * @param testName the test name
     * @param bladeId the blade id
     * 
     * @return the max time for
     */
    public long getMaxTimeFor(String testName, String bladeId);
    
    
    /**
     * Get the running life-cycle for a blade in a test 
     * @param testName the test name 
     * @param bladeId the blade ID
     */
    public long getLifecycleRunning(String testName, String bladeId);
    
    
    /**
     * Get the completed life-cycle for a blade in a test 
     * @param testName the test name 
     * @param bladeId the blade ID
     */
    public long getLifecycleCompleted(String testName, String bladeId);
    
    
    /**
     * Get all the values for a field from an event type, bladeId and test name
     * @param testName the test name 
     * @param bladeId the blade ID
     * @param eventTypeLabel the event type name/label
     * @param XFieldLabel the X field
     * @param YFieldLabel the Y field
     * @param start the start time
     * @param end the end time
     * @param count the values count 
     * @return an object[][] that contains the values
     */
    public Object[][] getAllFieldsValues(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count);
    
    
    /**
     * Get a bladeEvent array from a testName, a bladeId, an event
     */
    public BladeEvent[] getBladeEvents(String testName,	String bladeId, String eventTypeLabel,EventFilter filter, long fromIndex,int count)throws ClifException;

    
    /**
     * Get 2 columns from a bladeEvent[] : use first getBladeEvents
     */
    public Object[][] getBladeEventsValues(BladeEvent[] bladeEventList, String XFieldLabel, String YFieldLabel);
    	
    
    /**
     * Get the blade descriptor from a test and a blade filter
     * @param testName
     * @param filter
     */
    public BladeDescriptor[] getBladeDescriptor(String testName, BladeFilter filter);
}
