/*
* CLIF is a Load Injection Framework
* Copyright (C) 2008 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.analyze.api.graph;

import java.util.List;
import java.util.Map;
import org.jfree.chart.JFreeChart;
import org.ow2.clif.analyze.lib.graph.ChartData;
import org.ow2.clif.analyze.lib.graph.DatasetOptions;
import org.ow2.clif.analyze.lib.graph.StatisticDataImpl;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.supervisor.api.ClifException;

/**
	@author Olivier Liu
*/

public interface VirtualDataSupplier{
	 //  Fractal interface name 
    /** The Constant FILTER_DATA_CONTROL. */
    static public final String FILTER_DATA_CONTROL = "Filter Data Control";
    
    /** The Constant AGGREGATE_DATA_CONTROL. */
    static public final String AGGREGATE_DATA_CONTROL = "Aggregate Data Control";
    
    /** The Constant STATISTIC_DATA_CONTROL. */
    static public final String STATISTIC_DATA_CONTROL = "Statistic Data Control";
    
    /** The Constant ROUTINE_SUPPLIER. */
    static public final String ROUTINE_SUPPLIER = "Routine Supplier";
    
    /** The Constant VIRTUAL_DATA_SUPPLIER. */
    static public final String VIRTUAL_DATA_SUPPLIER = "Virtual Data supplier";
    
    ////////////////////////////////////
    // interface AggregateDataControl //
    ////////////////////////////////////	
    /**
     * Aggregate blades on one event type. The blades must have same type
     * (i.e. injectors, or probes of the same kind/class)
     * @return List<BladeEvent> virtual data from an aggregation on one event of all the blades selected
     */
    public List<BladeEvent>  aggregateBladesOneSameEvent(List<List<BladeEvent>> bladeList);
    
    /////////////////////////////////
    // interface FilterDataControl //
    /////////////////////////////////	
    /**
	 * Get the common minimum time for a given test(last minimum time for all blades for the given test in lifecycle)
	 * 
	 * @param testName the test name
	 * @return the common minimum time 
	 */
    public long getCommonMinTime(String testName);
    
    
    /**
	 * Get the common maximum time for a given test(last maximum time for all blades for the given test in lifecycle)
	 *  
	 * @param testName the test name
	 * @return the common maximum time 
	 */
    public long getCommonMaxTime(String testName);
    
    
	/** 
	 * Get a blade event list
	 * 
	 * @param testName the test name
	 * @param bladeId the blade Id
	 * @param eventTypeLabel the event Type
	 * @return a BladeEvent List
	 * @throws ClifException
	 */
	public List<BladeEvent> getBladeOneEventValues(
			String testName,	
			String bladeId, 
			String eventTypeLabel
			)throws ClifException;

	
    /**
	 * Filter the event from a blade with many filters
	 * 
	 * @param testName
	 * @param bladeId
	 * @param eventTypeLabel
	 * @param filter 
	 * @param linkingOp the linking operator
	 * @return a BladeEvent List
	 */
    public List<BladeEvent> getBladeAllEventsValuesFiltered(
    		String testName, 
    		String bladeId, 
    		String eventTypeLabel,
    		Map<String, Map<String, String>> filter,
    		String linkingOp);
    

	/**
	 * Get all the events for the blade
	 * 
	 * @param testName
	 * @param bladeId
	 * @param eventTypeLabel
	 * @return a Map<String, List<BladeEvent>> blade events 
	 * @throws ClifException
	 */
	public Map<String, List<BladeEvent>> getBladeAllEventsValues(
			String testName, 
			String bladeId, 
			String eventTypeLabel
			)throws ClifException;
	
	

    /**
     * Get a blade event list for a test, blade and event
     * 
     * @param testName
     * @param bladeId
     * @param eventTypeLabel
     * @param fieldLabel
     * @param fieldFilterKeyword
     * @param filterOperator
     * @return bladeEvent list
     */
    public List<BladeEvent> fieldFilter(
    		String testName, 
    		String bladeId, 
    		String eventTypeLabel, 
    		String fieldLabel, 
    		String fieldFilterKeyword, 
    		String filterOperator);
    
    
    /**
     * Filter with a bladeEvent list
     * 
     * @param bladeEventList
     * @param fieldLabel
     * @param fieldFilterKeyword
     * @param filterOperator
     * @return bladeEvent list
     */
    public List<BladeEvent> fieldFilter(
    		List<BladeEvent> bladeEventList,
    		String fieldLabel, 
    		String fieldFilterKeyword, 
    		String filterOperator
    );
    
    
    /**
     * Filter with many filters
     * 
     * @param bladeEventList
     * @param filter
     * @param linkingOp
     * @return bladeEvent list
     */
    public List<BladeEvent> getBladeAllEventsValuesFiltered(
    		List<BladeEvent> bladeEventList,
    		Map<String, Map<String, String>> filter,
    		String linkingOp);
    
    
    /**
     * Get bladeEvent list filtered by time
     * 
     * @param bladeEventList
     * @param start
     * @param end
     * @return bladeEvent list
     */
    public List<BladeEvent> getBladeByTime(
    		List<BladeEvent> bladeEventList,
			int start, 
			int end);
    
    	  
    /**
     * Get blade event array from a test, blade and event
     */
    public BladeEvent[] getBladeEvents(
    		String testName,	
    		String bladeId, 
    		String eventTypeLabel,
    		EventFilter filter, 
    		long fromIndex,
    		int count
    		)throws ClifException;

    
    ////////////////////////////////////
    // interface AnalysisRoutineWrite //
    ////////////////////////////////////
    /**
     * Save routine to XML
     * 
     * @param fileName
     * @param statisticMap
     * @param commentMap
     * @param jFreeChartMap
     * @param imageType
     * @param pageIdAndDatasetIds
     * @param chartDataList
     * @param datasetIdAndGraphOptions
     * @return boolean : true if successful
     */
    public boolean saveRoutineToXML(
    		String fileName, 
    		Map<Integer, List<StatisticDataImpl>> statisticMap,
    		Map<String, String> commentMap,
    		Map<Integer, JFreeChart> jFreeChartMap,
    		String imageType,
    		Map<Integer, List<Integer>> pageIdAndDatasetIds,
    		Map<Integer, ChartData> chartDataList,
    		Map<Integer, DatasetOptions> datasetIdAndGraphOptions);
    
}
