/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2008 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */
package org.ow2.clif.analyze.api.graph.virtualdata;

import java.util.List;
import java.util.Map;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.supervisor.api.ClifException;


/**
 * The Interface FilterDataControl.
 * @author Olivier Liu
 */
public interface FilterDataControl {

	//  Fractal interface name 
	/** The Constant FILTER_DATA_CONTROL. */
	static public final String FILTER_DATA_CONTROL = "Filter Data Control";

	/////////////////////////////////
	// Functions validated  //
	/////////////////////////////////

	/** 
	 * Get a blade event list
	 * 
	 * @param testName the test name
	 * @param bladeId the blade Id
	 * @param eventTypeLabel the event Type
	 * @return a BladeEvent List
	 * @throws ClifException
	 */
	public List<BladeEvent> getBladeOneEventValues(
			String testName,	
			String bladeId, 
			String eventTypeLabel
			)throws ClifException;

	
	/**
	 * Get all the events for the blade
	 * 
	 * @param testName
	 * @param bladeId
	 * @param eventTypeLabel
	 * @return a Map<String, List<BladeEvent>> blade events 
	 * @throws ClifException
	 */
	public Map<String, List<BladeEvent>> getBladeAllEventsValues(
			String testName,
			String bladeId, 
			String eventTypeLabel
			)throws ClifException;


	/////////////////////////////////
	// Functions under test  //
	/////////////////////////////////
	
	/**
	 * Filter the event from a blade with many filters
	 * 
	 * @param testName
	 * @param bladeId
	 * @param eventTypeLabel
	 * @param filter 
	 * @param linkingOp the linking operator
	 * @return a BladeEvent List
	 */
	public List<BladeEvent> getBladeAllEventsValuesFiltered(
    		String testName, 
    		String bladeId, 
    		String eventTypeLabel,
    		Map<String, Map<String, String>> filter,
    		String linkingOp);
	
	
	public BladeEvent[] getBladeEvents(
			String testName,	
			String bladeId, 
			String eventTypeLabel,
			EventFilter filter, 
			long fromIndex,
			int count)throws ClifException;


	public Object[][] getBladeEventsValues(
			BladeEvent[] bladeEventList, 
			String XFieldLabel, 
			String YFieldLabel);

	
	/**
	 * Filter blade by a field value
	 *
	 * @param bladeId the blade ID
	 * @param eventTypeLabel the label of an event type
	 * @param fieldLabel the field label
	 * @param fieldFilterKeyword the field filter keyword
	 * @param filterOperator the filter operator
	 * @return a virtual blade filtered by field
	 */
	public List<BladeEvent> fieldFilter(
			String testName, 
			String bladeId, 
			String eventTypeLabel, 
			String fieldLabel, 
			String fieldFilterKeyword, 
			String filterOperator);

	
	/**
	 * Get the common minimum time for a given test(last minimum time for all blades for the given test in lifecycle)
	 * 
	 * @param testName the test name
	 * @return the common minimum time 
	 */
	public long getCommonMinTime(String testName);

	
	/**
	 * Get the common maximum time for a given test(last maximum time for all blades for the given test in lifecycle)
	 *  
	 * @param testName the test name
	 * @return the common maximum time 
	 */
	public long getCommonMaxTime(String testName);

	
	/**
	 * Filtered with or operator
	 * 
	 * @param testName
	 * @param bladeId
	 * @param eventTypeLabel
	 * @param bladeEventListList
	 * @param linkingOp
	 * @return bladeEvent list
	 */
	public List<BladeEvent> getBladeAllEventsValuesFilteredOr(
	    		String testName,
	    		String bladeId,
	    		String eventTypeLabel,
	    		List<List<BladeEvent>> bladeEventListList,
	    		String linkingOp);


	/**
	 * Filter all events
	 * 
	 * @param bladeEventList
	 * @param filter
	 * @param linkingOp
	 * @return bladeEvent list
	 */
	public List<BladeEvent> getBladeAllEventsValuesFiltered(
			List<BladeEvent> bladeEventList,
			Map<String, Map<String, String>> filter, String linkingOp);


	/**
	 * Filter bladeEvent list
	 * @return bladeEvent list filtered
	 */
	public List<BladeEvent> fieldFilter(List<BladeEvent> bladeEventList,
			String fieldLabel, String fieldFilterKeyword, String filterOperator);

	
	/**
	 * Filter blade by a time interval
	 */
	public List<BladeEvent> getBladeByTime(List<BladeEvent> bladeEventList, int start, int end);


	/////////////////////////////////
	// Functions to do   //
	/////////////////////////////////
	
	/**
	 * Filter blade by a factor K filter
	 * 
	 * @param bladeId the blade ID
	 * @param factorKFilter the factor K
	 */
//	public /*VirtualBlade*/void sortFactorFilter(String testName, String bladeId, int factorKFilter);


	/**
	 * Filter blade by a sort ratio filter
	 * 
	 * @param bladeId the blade ID
	 * @param sortRatioFilter the sort ratio filter
	 */
//	public /*VirtualBlade*/void sortRatioFilter(String testName, String bladeId, double sortRatioFilter);









}
