/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name: not supported by cvs2svn $
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.probe.util;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.ArrayList;
import java.util.StringTokenizer;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.ow2.clif.datacollector.api.DataCollectorWrite;
import org.ow2.clif.datacollector.lib.GenericFilter;
import org.ow2.clif.scenario.isac.util.BooleanHolder;
import org.ow2.clif.server.api.BladeControl;
import org.ow2.clif.server.api.BladeInsertResponse;
import org.ow2.clif.server.util.EventStorageState;
import org.ow2.clif.storage.api.AlarmEvent;
import org.ow2.clif.storage.api.ProbeEvent;
import org.ow2.clif.supervisor.api.ClifException;


/**
 * This abstract implementation of a blade insert component manages a single thread,
 * doing a periodic task (probing) with a given period (in milliseconds) and during
 * an given actual time. It provides a "dumb" interpretation of the blade lifecycle
 * with regard to the ActivityControl interface, inasmuch as it actually probes from
 * its initialization time to its stop time, regardless of start(), suspend() and
 * resume() orders.
 * @author Bruno Dillenseger
 * @author Emmanuel Varoquaux
 */
abstract public class AbstractDumbInsert
	implements
		BladeControl,
		BindingController,
		LifeCycleController,
		Runnable
{
	static private final String[] interfaceNames = new String[] {
		DataCollectorWrite.DATA_COLLECTOR_WRITE,
		BladeInsertResponse.BLADE_INSERT_RESPONSE };
	protected String probeId;
	protected Object activity_lock = new Object();
	protected BladeInsertResponse bir;
	protected Object sr_lock = new Object();
	protected DataCollectorWrite dcw;
	protected Object dc_lock = new Object();
	protected long arg_period_ms = 0;
	protected long arg_duration_ms = 0;
    protected ArrayList arg_probe_config = new ArrayList();
	protected long baseTime_ms = 0;
	protected volatile boolean started;
	protected volatile boolean stopped;
	protected volatile boolean suspended;
	protected volatile boolean terminated;
	private String fcState = LifeCycleController.STOPPED;
	protected Thread poller;

	protected final BooleanHolder storeLifeCycleEvents = new BooleanHolder(true);
	protected final BooleanHolder storeAlarmEvents = new BooleanHolder(true);
	protected final BooleanHolder storeProbeEvents = new BooleanHolder(true);
	protected final Map eventStorageStatesMap = new HashMap();

	/**
	 * Override this method to perform your measure and return it
	 * @return an probe event holding the measure
	 */
	abstract protected ProbeEvent doProbe();
    
    /** 
     * Override this method when you are willing to pass some parameters to the Insert.
     * Parameters may be an xml config file, the specification of a disk name or a network interface.
     * The parameters have to be entered on the BladeArgument section of the clif gui
     * after the arg_period_ms and arg_duration_ms parameters.
     */
    protected void configProbewithArray() throws ClifException
    {
    }

    /** 
     * Override this method when you are willing to release some resources
     * when the probe ends its activity.
     */
    protected void close() throws ClifException
    {
    }
    
	///////////////////////////////////
	// interface LifeCycleController //
	///////////////////////////////////


	public void startFc()
	{
		fcState = LifeCycleController.STARTED;
	}


	public void stopFc()
	{
		fcState = LifeCycleController.STOPPED;
	}


	public String getFcState()
	{
		return fcState;
	}


	///////////////////////////////////////////////////
	// implementation of interface BindingController //
	///////////////////////////////////////////////////


	public Object lookupFc(String clientItfName)
	{
		if (clientItfName.equals(DataCollectorWrite.DATA_COLLECTOR_WRITE))
		{
			return dcw;
		}
		else if (clientItfName.equals(BladeInsertResponse.BLADE_INSERT_RESPONSE))
		{
			return bir;
		}
		else
		{
			return null;
		}
	}


	public void bindFc(String clientItfName, Object serverItf)
	{
		if (clientItfName.equals(DataCollectorWrite.DATA_COLLECTOR_WRITE))
		{
			synchronized (dc_lock)
			{
				dcw = (DataCollectorWrite) serverItf;
			}
		}
		else if (clientItfName.equals(BladeInsertResponse.BLADE_INSERT_RESPONSE))
		{
			synchronized (sr_lock)
			{
				bir = (BladeInsertResponse) serverItf;
			}
		}
	}


	public void unbindFc(String clientItfName)
	{
		if (clientItfName.equals(DataCollectorWrite.DATA_COLLECTOR_WRITE))
		{
			synchronized (dc_lock)
			{
				dcw = null;
			}
		}
		else if (clientItfName.equals(BladeInsertResponse.BLADE_INSERT_RESPONSE))
		{
			synchronized (sr_lock)
			{
				bir = null;
			}
		}
	}


	public String[] listFc()
	{
		return interfaceNames;
	}


	//////////////////////////////////////////////
	// implementation of interface BladeControl //
	//////////////////////////////////////////////


	/**
	 * @param testId unique identifier of the new test
	 */
	public void init(Serializable testId)
	{
		started = false;
		stopped = false;
		suspended = false;
		terminated = false;
		poller = new Thread(this, "CLIF probe");
		poller.start();
	}


	public void start()
	{
		started = true;
		baseTime_ms = System.currentTimeMillis();
	}


	public void stop()
	{
		stopped = true;
		poller.interrupt();
		synchronized (activity_lock)
		{
			if (started && ! terminated)
			{
				try
				{
					activity_lock.wait();
				}
				catch (InterruptedException ex)
				{
				}
			}
		}
	}


	public void suspend()
	{
		suspended = true;
		baseTime_ms = System.currentTimeMillis() - baseTime_ms;
	}


	public void resume()
	{
		baseTime_ms = System.currentTimeMillis() - baseTime_ms;
		suspended = false;
	}


	public void join()
	{
		// no implementation because join() is caught/implemented by the blade insert adapter
		throw new Error("A blade insert's join() method should never be called (call its adapter's join() method instead)");
	}

	/**
	 * Sets number of threads and test duration parameters
	 * @param arg should hold 2 integer parameters (separated with usual separators) setting
	 * (1) the observation polling period in ms, and (2) the test duration in seconds.
	 */
	public void setArgument(String arg)
		throws ClifException
	{
		StringTokenizer parser = new StringTokenizer(arg);
		try
		{     
			arg_period_ms = Integer.parseInt(parser.nextToken());
			arg_duration_ms = Integer.parseInt(parser.nextToken()) * 1000;
		}
		catch (Exception ex)
		{
			throw new ClifException("probe expects at least 2 arguments: <period_ms> <duration_s>");
		}
		// Probe inner configuration will be done thanks to ArrayList arg_probe_config
        arg_probe_config.clear();
		while (parser.hasMoreTokens()) {
        	arg_probe_config.add (parser.nextToken(";").replaceFirst(" ",""));
        }
        configProbewithArray();
	}


	/**
	 * Sets this scenario's unique identifier
	 */
	public void setId(String id)
	{
		probeId = id;
	}


	/**
	 * @return the scenario/blade identifier
	 */
	 public String getId()
	{
		return probeId;
	}


	public void run()
	{
		while (!stopped && (!started || suspended || arg_duration_ms > System.currentTimeMillis() - baseTime_ms))
		{
			try
			{
				dcw.add(doProbe());
			}
			catch (Exception ex)
			{
				bir.alarm(new AlarmEvent(
					System.currentTimeMillis(),
					AlarmEvent.WARNING,
					new ClifException(
						"Exception in probe " + probeId + " while probing",
						ex)));
			}
			try
			{
				long time = System.currentTimeMillis();
				if (!started || suspended || arg_duration_ms > time - baseTime_ms + arg_period_ms)
				{
					Thread.sleep(arg_period_ms);
				}
				else
				{
					long sleep = arg_duration_ms - time + baseTime_ms;
					if (sleep > 0)
					{
						Thread.sleep(sleep);
					}
				}
			}
			catch (InterruptedException ex)
			{
			}
		}
		synchronized (activity_lock)
		{
			terminated = true;
			try
			{
				close();
			}
			catch(Exception ex)
			{
				bir.alarm(new AlarmEvent(
					System.currentTimeMillis(),
					AlarmEvent.WARNING,
					new ClifException(
						"Exception in probe " + probeId + " while terminating",
						ex)));
			}
			if (! stopped)
			{
				bir.completed();
			}
			else
			{
				activity_lock.notify();
			}
		}
	}

	public void changeParameter(String parameter, Serializable value) throws ClifException {
		if (EventStorageState.setEventStorageState(eventStorageStatesMap, parameter, value)) {
			dcw.setFilter(new GenericFilter(false, storeAlarmEvents.getBooleanValue(), storeLifeCycleEvents.getBooleanValue(), storeProbeEvents.getBooleanValue()));
			return;
		}
	}

	public Map getCurrentParameters() {
		Map parameters = new HashMap();

		EventStorageState.putEventStorageStates(parameters, eventStorageStatesMap);
		return parameters;
	}
}
