/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004, 2010 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.server.lib;

import java.net.InetAddress;
import java.util.HashMap;
import java.util.Map;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.util.Fractal;
import org.ow2.clif.deploy.ClifRegistry;
import org.ow2.clif.server.api.ClifServerControl;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.supervisor.api.SupervisorInfo;
import org.ow2.clif.util.ClifClassLoader;
import org.ow2.clif.util.ExecutionContext;


/**
 * Manages blade creation and removal within a given Clif server.
 * @author Bruno Dillenseger
 */
public class ClifServerImpl
	implements ClifServerControl
{
	/**
	 * Creates a CLIF server in current/local Fractal environment, and
	 * binds it with the given name in the given registry. If this name
	 * is already bound, the existing binding is overriden by this one.
	 * @param name bound name in the registry
	 * @param registry registry where to bind this new CLIF server
	 */
	static public void create(String name, ClifRegistry registry)
		throws ClifException
	{
		try
		{
			Component bootstrap = Fractal.getBootstrapComponent();
			Factory factory = FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND);
			ComponentType serverType = (ComponentType) factory.newComponentType(
				"org.ow2.clif.server.api.ClifServerType",
				null);
			Component clifServer = Fractal.getGenericFactory(bootstrap).newFcInstance(
				serverType,
				"primitive",
				"org.ow2.clif.server.lib.ClifServerImpl");
			Fractal.getLifeCycleController(clifServer).startFc();
			registry.bindServer(name, clifServer);
		}
		catch (Exception ex)
		{
			throw new ClifException("Could not create ClifServer " + name, ex);
		}
	}


	/**
	 * Creates a CLIF server and binds it in the CLIF registry
	 * @param args if no argument is given, the CLIF server is bound to a default name
	 * that is the network name of the local environment. Otherwise, the new CLIF server
	 * is bound to the name given by the first argument (i.e. args[0]). 
	 */
	static public void main(String[] args)
		throws Exception
	{
		if (System.getSecurityManager() == null)
		{
			System.setSecurityManager(new SecurityManager());
		}
		ExecutionContext.init("./");
		String name = args.length == 0 ? InetAddress.getLocalHost().getHostName() : args[0];
		create(
			name,
			new ClifRegistry(false));
		System.out.println("CLIF server " + name + " is ready.");
	}


	/** contains blade components of this CLIF server, indexed by their identifier */
	protected Map<String, Component> blades = new HashMap<String, Component>();


	public ClifServerImpl()
	{
	}


	/////////////////////////////////
	// interface ClifServerControl //
	/////////////////////////////////


	/**
	 * Creates a new blade component managed by this Clif server from an ADL definition,
	 * @param adlDefinition ADL definition name of a blade implementation
	 * @param adlParams ADL parameters that will be used for blade instantiation
	 * (parameters used by the ADL definition)
	 * @param bladeId identifier for the new blade (used to name the new blade component
	 * with regard to Fractal NameController)
	 * @return the BladeControl interface of the new Blade component
	 * @throws ClifException
	 */
	public synchronized Component addBlade(Component clifApp, String adlDefinition, Map adlParams, String bladeId)
		throws ClifException
	{
		try
		{
			Map context = new HashMap();
			context.putAll(adlParams);
			context.put("classloader", ClifClassLoader.getClassLoader());
			Factory adlFactory = FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND, context);
			Component blade = (Component)adlFactory.newComponent(adlDefinition, context);
			Fractal.getNameController(blade).setFcName(bladeId);
			Fractal.getContentController(clifApp).addFcSubComponent(blade);
			String clifAppName = Fractal.getNameController(clifApp).getFcName();
			blades.put(clifAppName+bladeId, blade);
			return blade;
		}
		catch (NoSuchInterfaceException ex)
		{
			ClifClassLoader.clear();
			throw new Error(
				"Bad configuration: unable to add blade " + bladeId + " to the Clif Application",
				ex);
		}
		catch (IllegalContentException ex)
		{
			ClifClassLoader.clear();
			throw new Error(
				"Bad configuration: unable to add blade " + bladeId + " to the Clif Application",
				ex);
		}
		catch (IllegalLifeCycleException ex)
		{
			ClifClassLoader.clear();
			throw new Error(
				"Bad configuration: unable to add blade " + bladeId + " to the Clif Application",
				ex);
		}
		catch (ADLException ex)
		{
			ClifClassLoader.clear();
			throw new ClifException("Unknown class for blade " + bladeId + " " + adlParams, ex);
		}
		catch (Exception ex)
		{
			ClifClassLoader.clear();
			throw new ClifException("Unable to deploy blade " + bladeId, ex);
		}
	}


	/**
	 * Unbinds and removes every blade managed by this CLIF server,
	 * and clears the CLIF class loader cache.
	 * @throws ClifException if at least one blade could not be removed.
	 * Note: all removable blades are removed before throwing the exception,
	 * and the CLIF class loader cache is cleared anyway.
	 */
	public synchronized void removeAllBlades(Component clifApp)
		throws ClifException
	{
		ClifException exception = null;
		String[] ids = blades.keySet().toArray(new String[blades.size()]);
		for (int i=0 ; i<ids.length ; ++i)
		{
			try
			{
				removeBlade(clifApp, ids[i]);
			}
			catch (ClifException ex)
			{
				exception = ex;
			}
		}
		if (exception != null)
		{
			ClifClassLoader.clear();
			throw exception;
		}	
	}


	/**
	 * Unbinds the SupervisorInfo interface and removes a blade from both this Clif server and
	 * the Clif application it belongs to. Assumes that the blade is stopped. Once this blade
	 * is removed, the CLIF class loader cache is cleared if and only if there is no other
	 * blade remaining in this CLIF server.
	 * @param clifApp the Clif application distributed component the blade belongs to
	 * @param id blade identifier
	 * @throws ClifException if this blade could not be correctly removed, either because
	 * it is unknown by this CLIF server, or because a technical problem occurred in the
	 * distributed infrastructure management. The exception embedded in the ClifException
	 * gives more details about the root cause.
	 */
	public synchronized void removeBlade(Component clifApp, String id)
		throws ClifException
	{
	    Component blade = blades.remove(id);
	    if (blade == null)
	    {
	        throw new ClifException("Unknown blade identifier " + id);
	    }
	    
	    try{
	        if(id.startsWith(Fractal.getNameController(clifApp).getFcName()))
	        {
	            Fractal.getBindingController(blade).unbindFc(SupervisorInfo.SUPERVISOR_INFO);
	            Fractal.getContentController(clifApp).removeFcSubComponent(blade);
	            if (blades.isEmpty())
	            {
	                ClifClassLoader.clear();
	            }
	        }
	    }
		catch (Exception ex)
		{
			blades.put(id, blade);
			String message = "Unable to complete removal of blade " + id;
			throw new ClifException(message, ex);
		}
	}
}
