/*
* CLIF is a Load Injection Framework
* Copyright (C) 2006, 2009, 2010 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * This class helps gathering configuration parameters to ease portability between
 * several assemblies of CLIF. It also configures log libraries and system properties.
 * 
 * @author Bruno Dillenseger
 * @author Florian Francheteau
 */
public abstract class ExecutionContext
{
	/**
	 * Name of the default CLIF server possibly launched by a user
	 * interface for convenience
	 */
	static public final String DEFAULT_SERVER = "local host";

	/**
	 * Base directory of CLIF runtime environment (may be a relative or absolute filesystem path).
	 * Must end with a filesystem path separator.
	 */
	static private String baseDir = null;
	
	/**
	 * absolute path to the file holding MaxQ properties in current project.
	 */
	static private String maxQPropFile = null;

	/**
	 * relative path to the template file for CLIF system properties
	 */
	static public final String TEMPLATE_PATH = "etc/clif.props.template";
	
	/**
	 * relative path to the file holding MaxQ properties in console plugin
	 */
	static public final String DEFAULT_MAXQ_PROPERTY_FILE = "etc/maxq.properties";

	/**
	 * relative path to the file holding CLIF system properties
	 */
	static public final String PROPS_PATH = "etc/clif.props";
	
	/**
	 * absolute path to the workspace
	 */
	static public String WORKSPACE_PATH;

	/**
	 * Names of CLIF system properties whose value is a path
	 */
	static public String[] PROPERTIES_WITH_PATH = {
		"java.security.policy","julia.config","clif.codeserver.path","clif.filestorage.dir"};


	/**
	 * Sets the base directory of CLIF runtime environment, and configures log libraries.
	 * @param path the filesystem path to the base directory of CLIF runtime environment.
	 * Must have a trailing filesystem separator.
	 */
	static public void init(String path)
	{
		File pathFile = new File(path);
		if (pathFile.isAbsolute())
		{
			baseDir = path;
		}
		else
		{
			baseDir = pathFile.getAbsolutePath() + File.separator;
		}
	}


	/**
	 * Gets the base directory of CLIF runtime environment.
	 * @return the base directory of CLIF runtime environment, or null if this directory
	 * has not been set.
	 */
	static public String getBaseDir()
	{
		return baseDir;
	}


	/**
	 * Sets system properties according to the contents of an input stream.
	 * The stream is read as a proerties-formatted content, from which
	 * property clif.parameters is analyzed and transformed into system
	 * properties.
	 * @param ins input stream properties are read from
	 */
	static public void setProperties(InputStream ins)
	{
		try
		{
			Properties props = new Properties();
			props.load(ins);
			String allParam = (String)props.get("clif.parameters");
			String[] parameters = allParam.split(" ");
			for (int i = 0; i < parameters.length; i++)
			{
				if(parameters[i].startsWith("-D"))
				{
					String[] property = parameters[i].replaceFirst("-D","").split("=");
					System.setProperty(property[0], property[1]);
					for (int j = 0; j < PROPERTIES_WITH_PATH.length; j++)
					{
						if(PROPERTIES_WITH_PATH[j].equals(property[0]))
						{
							if((!"clif.filestorage.dir".equals(property[0]))||(WORKSPACE_PATH==null))
								System.setProperty(property[0], completePath(property[0]));
							break;
						}
					}
				}
			}
			props.remove("clif.parameters");
			for (Object key : props.keySet())
			{
				System.setProperty(
					(String)key,
					(String)props.getProperty((String)key));
			}
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}


	static private String completePath(String property)
	{
		String path = "";
		try
		{
			String[] paths = System.getProperty(property).replaceAll("\"","").split(";");
			for (int i = 0; i < paths.length-1; i++)
			{
				path += foundPath(paths[i]) + ";";
			}
			path += foundPath(paths[paths.length-1]);
			return path;
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		return path;
	}


	/**
	 * @param path
	 * @return String the correct path
	 * @throws IOException
	 */
	static private String foundPath(String path) throws IOException
	{
		if (new File(path).isAbsolute())
		{
			return path;
		}
		else
		{
			return getBaseDir() + path;
		}
	}
	
	
	/**
	 * Sets absolute location of maxq.properties file in current project
	 * @param dir absolute location of maxq.properties
	 */
	static public void setMaxQPropFile(String dir){
		maxQPropFile=dir;
	}
	
	/**
	 * Gets location of maxq.properties file in current project
	 * @return String 	the absolute path of maxq.properties
	 */
	static public String getMaxQPropFile(){
		return maxQPropFile;
	}
}
