/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004, 2005, 2010 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.util;

import java.io.IOException;
import java.net.Inet4Address;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * This utility class provides a method to get all IPv4 addresses of local network
 * interfaces, either randomly or in a round-robin way.
 * If at least an IPv4 address is configured, then loopback/localhost addresses are
 * ignored. Otherwise, if no network interface holds any configured IPv4 address,
 * a default localhost address will be selected. IPv6 addresses are ignored anyway.
 * @author Bruno Dillenseger
 */
abstract public class Network
{
	static protected InetAddress[] inetAddresses = null;
	static private int nextInetAddressIndex = 0;
	static Random random = new Random();

	static {
		try
		{
			ArrayList<InetAddress> allAddresses = new ArrayList<InetAddress>();
			// lookup all network interfaces
			
			Enumeration<NetworkInterface> interfaces = NetworkInterface.getNetworkInterfaces();
			while (interfaces.hasMoreElements())
			{
				// lookup all IP addresses
				Enumeration<InetAddress> addresses = interfaces.nextElement().getInetAddresses();
				while (addresses.hasMoreElements())
				{
					InetAddress address = addresses.nextElement();
					// discard loopback and IPv6 addresses
					if (!address.isLoopbackAddress() && ! (address instanceof Inet6Address))
					{
						allAddresses.add(address);
					}
				}
			}
			if (allAddresses.size() == 0)
			{
				// in case no IP address is configured, get loopback address
				inetAddresses = new InetAddress[] { InetAddress.getLocalHost() };
			}
			else
			{
				inetAddresses = allAddresses.toArray(new InetAddress[allAddresses.size()]);
			}
		}
		catch (Exception ex)
		{
			throw new Error("Could not get a network address on this host", ex);
		}
		String addresses = "available InetAddresses:";
		for (InetAddress address : inetAddresses)
		{
			addresses += " " + address;
		}
	}

	
	/**
	 * @param subnet if not null, this string must specify an IPv4 subnetwork address
	 * in the form b1.b2.b3.b4/maskbits (e.g. 192.168.1.0/24)
	 * @return an IP address picked among available local IPv4 addresses, 
	 * in a round-robin way, belonging to the specified subnet if not null.
	 * 
	 */
	static public synchronized InetAddress getNextInetAddress(String subnet)
	{
		InetAddress result = null;
		if (inetAddresses != null)
		{
			if (nextInetAddressIndex == inetAddresses.length)
			{
				nextInetAddressIndex = 0;
			}
			if (subnet == null)
			{
				return inetAddresses[nextInetAddressIndex++];	
			}
			else
			{
				while (nextInetAddressIndex < inetAddresses.length && result == null)
				{
					if (belongsToSubnet((Inet4Address)inetAddresses[nextInetAddressIndex], subnet))
					{
						result = inetAddresses[nextInetAddressIndex];
					}
					++nextInetAddressIndex;
				}
			}
		}
		return result;
	}


	/**
	 * @return an IP address randomly picked among available local IPv4 addresses
	 */
	static public synchronized InetAddress getRandomInetAddress()
	{
		if (inetAddresses == null)
		{
			return null;
		}
		else
		{
			nextInetAddressIndex = random.nextInt(inetAddresses.length);
			return inetAddresses[nextInetAddressIndex++];
		}
	}


	/**
	 * @param subnet if not null, this string must specify an IPv4 subnetwork address
	 * in the form b1.b2.b3.b4/maskbits (e.g. 192.168.1.0/24)
	 * @return an arbitrary local IPv4 address belonging to the specified subnet
	 * if not null. Successive calls always return the same address.
	 */
	static public InetAddress getInetAddress(String subnet)
	{
		InetAddress result = null;
		if (inetAddresses != null)
		{
			if (subnet == null)
			{
				result = inetAddresses[0];
			}
			else
			{
				for (int i=0 ; i<inetAddresses.length && result == null ; ++i)
				{
					if (belongsToSubnet((Inet4Address)inetAddresses[i], subnet))
					{
						result = inetAddresses[i];
					}
				}
			}
		}
		return result;
	}


	/**
	 * @param subnet if not null, this string must specify an IPv4 subnetwork address
	 * in the form b1.b2.b3.b4/maskbits (e.g. 192.168.1.0/24)
	 * @return all available local IPv4 addresses, or null if no IPv4 address could
	 * be found. If subnet parameter is not null, only addresses belonging to the
	 * specified subnet are returned.
	 */
	static public InetAddress[] getInetAddresses(String subnet)
	{
		if (subnet == null)
		{
			return inetAddresses;
		}
		else
		{
			Set<InetAddress> addresses = new HashSet<InetAddress>(inetAddresses.length);
			for (InetAddress address : inetAddresses)
			{
				if (belongsToSubnet((Inet4Address)address, subnet))
				{
					addresses.add(address);
				}
			}
			return addresses.isEmpty()
				? null
				: addresses.toArray(new InetAddress[addresses.size()]);
		}
	}


	static public boolean belongsToSubnet(Inet4Address address, String subnet)
	{
		byte[] bytes = address.getAddress();
		long addressHash = 255 & bytes[0];
		int subnetBits = 32 - Integer.parseInt(subnet.substring(subnet.indexOf('/') + 1));
		StringTokenizer parser = new StringTokenizer(subnet, "./");
		long subnetHash = Integer.parseInt(parser.nextToken());
		for (int i=1 ; i<4 ; ++i)
		{
			addressHash = (addressHash << 8) + (255 & bytes[i]);
			subnetHash = (subnetHash << 8) + Integer.parseInt(parser.nextToken());
		}
		addressHash >>= subnetBits;
		subnetHash >>= subnetBits;
		return addressHash == subnetHash;
	}


	/**
	 * Prints to standard output the local IP address from
	 * which the provided host IP address or name is reachable.
	 * Keeps mute if no local address routes to the target host.
	 * Reports possible error messages to the standard error output.
	 * Exists with status code 0 if a correct local address was found,
	 * 1 if the target host is unreachable from any local address,
	 * or a greater value if an error occurred. 
	 * @param args 2 arguments expected:
	 * args[0] gives the target host IP address or name,
	 * args[1] gives the timeout in s for reachability assessment
	 * of each local address.
	 */
	static public void main(String[] args)
	{
		if (args.length != 2)
		{
			System.err.println("2 arguments expected: target_host_name_or_IP timeout_ms");
			System.exit(2);
		}
		try
		{
			InetAddress targetHost = InetAddress.getByName(args[0]);
			int timeout = new Integer(args[1]) * 1000;
			for (InetAddress addr : inetAddresses)
			{
				if (targetHost.isReachable(NetworkInterface.getByInetAddress(addr), 0, timeout))
				{
					System.out.println("network.address=" + addr.getHostAddress());
					System.exit(0);
				}
			}
		}
		catch (UnknownHostException ex)
		{
			System.err.println("Unknown host " + args[0]);
			System.exit(3);
		}
		catch (NumberFormatException ex)
		{
			System.err.println("Bad timeout value " + args[1]);
			System.exit(4);
		}
		catch (IOException ex)
		{
			System.err.println("Network error: " + ex);
			System.exit(5);
		}
		System.exit(1);
	}
}
