package org.ow2.isac.plugin.stringhandler;

import org.ow2.clif.scenario.isac.plugin.SessionObjectAction;

import java.util.HashMap;
import java.util.Hashtable;
import org.ow2.clif.scenario.isac.plugin.DataProvider;
import org.ow2.clif.scenario.isac.exception.IsacRuntimeException;
import java.lang.Error;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.ow2.clif.scenario.isac.plugin.TestAction;
import org.ow2.clif.scenario.isac.plugin.ControlAction;
import org.ow2.clif.util.Random;

/**
 * Implementation of a session object for plugin ~StringHandler~
 */
public class SessionObject implements SessionObjectAction, DataProvider, TestAction, ControlAction {

	static final String PLUGIN_DEFAULT = "default";
	static final String CONTROL_RANDOMSET_SIZE = "size";
	static final int CONTROL_SET = 0;
	static final String CONTROL_SET_VALUE = "value";
	static final int CONTROL_REPLACEALL = 1;
	static final String CONTROL_REPLACEALL_REPLACE = "replace";
	static final String CONTROL_REPLACEALL_SEARCH = "search";
	static final int CONTROL_RANDOMSET = 2;
	static final int CONTROL_SETPATTERN = 3;
	static final String CONTROL_SETPATTERN_PATTERN = "pattern";
	static final int CONTROL_RESETMATCHING = 4;
	static final int CONTROL_MATCH = 5;
	static final String CONTROL_MATCH_SKIP = "skip";
	static final int CONTROL_CAPTURE = 7;
	static final String CONTROL_CAPTURE_VARIABLE = "variable";
	static final String CONTROL_CAPTURE_LEVEL = "level";
	static final int CONTROL_CUT = 8;
	static final String CONTROL_CUT_TAIL = "tail";
	static final String CONTROL_CUT_HEAD = "head";
	static final int CONTROL_TRUNCATE = 11;
	static final String CONTROL_TRUNCATE_LENGTH = "length";
	static final int CONTROL_REPLACEALLMATCHES = 12;
	static final String CONTROL_REPLACEALLMATCHES_REPLACE = "replace";
	static final int CONTROL_WRITE = 13;
	static final String CONTROL_WRITE_STRING = "string";
	static final int CONTROL_APPEND = 14;
	static final String CONTROL_APPEND_STRING = "string";
	static final int CONTROL_INSERT = 15;
	static final String CONTROL_INSERT_STRING = "string";
	static final int TEST_CONTAINS = 0;
	static final String TEST_CONTAINS_SEARCH = "search";
	static final int TEST_MATCHED = 6;
	static final int TEST_CONTAINSNOT = 9;
	static final String TEST_CONTAINSNOT_SEARCH = "search";
	static final int TEST_MATCHEDNOT = 10;
	static private final String GET_LENGTH = "#";
	static private final String GET_STRING = "";

	static private Random random = new Random();

	private String defaultValue;
	private StringBuilder value;
	private Pattern pattern;
	private Matcher matcher;
	private Map<String,String> variables = new HashMap<String,String>();
	private boolean matched = false;

	/**
	 * Constructor for specimen object.
	 *
	 * @param params key-value pairs for plugin parameters
	 */
	public SessionObject(Hashtable params)
	{
		defaultValue = (String)params.get(PLUGIN_DEFAULT);
		if (defaultValue == null)
		{
			defaultValue = "";
		}
	}

	/**
	 * Copy constructor (clone specimen object to get session object).
	 *
	 * @param so specimen object to clone
	 */
	private SessionObject(SessionObject so)
	{
		value = new StringBuilder(so.defaultValue);
	}


	/**
	 * Must be called each time the string value changes
	 * to update a number of dependencies.
	 */
	private void valueChanged()
	{
		// reset pattern matcher
		if (pattern != null)
		{
			matcher.reset(value);
		}
	}


	////////////////////////////////////////
	// SessionObjectAction implementation //
	////////////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#createNewSessionObject()
	 */
	public Object createNewSessionObject() {
		return new SessionObject(this);
	}

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#close()
	 */
	public void close() {
		value = null;
	}

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#reset()
	 */
	public void reset() {
		pattern = null;
		matcher = null;
		variables = new HashMap<String,String>();
		matched = false;
	}


	/////////////////////////////////
	// DataProvider implementation //
	/////////////////////////////////


	/**
	 * @see org.ow2.clif.scenario.isac.plugin.DataProvider#doGet()
	 */
	public String doGet(String arg)
	{
		String result = null;
		if (arg.equals(GET_STRING))
		{
			result = value.toString();
		}
		else if (arg.equals(GET_LENGTH))
		{
			result = Integer.toString(value.length());
		}
		else
		{
			 result = variables.get(arg);
		}
		if (result == null)
		{
			throw new IsacRuntimeException("Unknown variable value in ~StringHandler~ ISAC plugin: " + arg);
		}
		else
		{
			return result;
		}
	}


	///////////////////////////////
	// TestAction implementation //
	///////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.TestAction#doTest()
	 */
	public boolean doTest(int number, Map params)
	{
		switch (number)
		{
			case TEST_MATCHEDNOT:
				return !matched;
			case TEST_CONTAINSNOT:
				return value.indexOf((String)params.get(TEST_CONTAINSNOT_SEARCH)) == -1; 
			case TEST_MATCHED:
				return matched;
			case TEST_CONTAINS:
				return value.indexOf((String)params.get(TEST_CONTAINS_SEARCH)) != -1;
			default:
				throw new Error("Unable to find this test in ~StringHandler~ ISAC plugin: " + number);
		}
	}

	
	//////////////////////////////////
	// ControlAction implementation //
	//////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.ControlAction#doControl()
	 */
	public void doControl(int number, Map params)
	{
		switch (number)
		{
			case CONTROL_INSERT:
				value.insert(0, (String)params.get(CONTROL_INSERT_STRING));
				break;
			case CONTROL_APPEND:
				value.append((String)params.get(CONTROL_APPEND_STRING));
				break;
			case CONTROL_WRITE:
				String overwrite = (String)params.get(CONTROL_WRITE_STRING);
				value.replace(0, overwrite.length(), overwrite);
				break;
			case CONTROL_REPLACEALLMATCHES:
				if (pattern != null)
				{
					value = new StringBuilder(matcher.replaceAll((String)params.get(CONTROL_REPLACEALLMATCHES_REPLACE)));
					valueChanged();
				}
				else
				{
					throw new IsacRuntimeException("StringHandler can't replace matches when no pattern is set.");
				}
				break;
			case CONTROL_TRUNCATE:
				int length = Integer.parseInt((String) params.get(CONTROL_TRUNCATE_LENGTH));
				if (length < value.length())
				{
					value.delete(length, value.length());
				}
				valueChanged();
				break;
			case CONTROL_CUT:
				int head = Integer.parseInt((String) params.get(CONTROL_CUT_HEAD));
				int tail = Integer.parseInt((String) params.get(CONTROL_CUT_TAIL));
				if (head > 0)
				{
					value.delete(0, head);
				}
				if (tail > 0)
				{
					value.delete(value.length()-tail, value.length());
				}
				valueChanged();
				break;
			case CONTROL_CAPTURE:
				if (matched)
				{
					variables.put(
						(String)params.get(CONTROL_CAPTURE_VARIABLE),
						matcher.group(
							Integer.parseInt((String)params.get(CONTROL_CAPTURE_LEVEL))));
				}
				else
				{
					variables.remove((String)params.get(CONTROL_CAPTURE_VARIABLE));
					throw new IsacRuntimeException("StringHandler plug-in can't capture an unmatched pattern.");
				}
				break;
			case CONTROL_MATCH:
				int i = Integer.parseInt((String)params.get(CONTROL_MATCH_SKIP));
				matched = false;
				while (i > 0 && matcher.find())
				{
					--i;
				}
				if (i == 0)
				{
					matched = matcher.find();
				}
				break;
			case CONTROL_RESETMATCHING:
				matcher.reset();
				matched = false;
				break;
			case CONTROL_SETPATTERN:
				pattern = Pattern.compile((String)params.get(CONTROL_SETPATTERN_PATTERN));
				matcher = pattern.matcher(value);
				matched = false;
				break;
			case CONTROL_RANDOMSET:
				value = random.nextStringBuilder(Integer.parseInt((String)params.get(CONTROL_RANDOMSET_SIZE)));
				valueChanged();
				break;
			case CONTROL_REPLACEALL:
				String search = (String)params.get(CONTROL_REPLACEALL_SEARCH);
				String replace = (String)params.get(CONTROL_REPLACEALL_REPLACE);
				int pos = 0;
				while (pos < value.length() && (pos = value.indexOf(search, pos)) != -1)
				{
					value.replace(pos, pos + search.length(), replace);
					pos += replace.length();
				}
				valueChanged();
				break;
			case CONTROL_SET:
				value = new StringBuilder((String)params.get(CONTROL_SET_VALUE));
				valueChanged();
				break;
			default:
				throw new Error("Unable to find this control in ~StringHandler~ ISAC plugin: " + number);
		}
	}
}
