/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.analyze.lib.oda;

import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;

import org.eclipse.datatools.connectivity.oda.IBlob;
import org.eclipse.datatools.connectivity.oda.IClob;
import org.eclipse.datatools.connectivity.oda.IResultSet;
import org.eclipse.datatools.connectivity.oda.IResultSetMetaData;
import org.eclipse.datatools.connectivity.oda.OdaException;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;

import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.analyze.api.AnalyzerExternalAccess;
import org.ow2.clif.deploy.ClifAnalyzerAppFacade;
import org.ow2.clif.analyze.lib.oda.i18n.Messages;
import org.ow2.clif.analyze.lib.oda.util.DateUtil;

/**
 * Flat file data provider's implementation of the ODA IResultSet interface.
 */

public class ResultSet implements IResultSet
{

    /** The Constant DEFAULT_MAX_ROWS. */
    public static final int DEFAULT_MAX_ROWS = 1000;
    private static final int CURSOR_INITIAL_VALUE = -1;
    private IResultSetMetaData resultSetMetaData = null;
    private long maxRows = DEFAULT_MAX_ROWS;
    private int cursor = CURSOR_INITIAL_VALUE;
    //Boolean which marks whether it is successful of last call to getXXX();
    private boolean wasNull = false;
    //a counter that counts the total number of rows read from the flatfile
    private int fetchAccumulator = 0;
    private BladeEvent result;
    private Serializable iteratorKey;
    private String testName;
    private String bladeId;
    private String eventTypeLabel;
    private EventFilter filter = null;
    private AnalyzerExternalAccess analyzer = null;
	private ClifAnalyzerAppFacade clifAnalyzerApp;
    
	
    /**
     * Constructor
     * @param rsmd
     */
    ResultSet(IResultSetMetaData rsmd )
    {
    	this.resultSetMetaData = rsmd;
    	clifAnalyzerApp = ClifAnalyzerAppFacade.getInstance();
    	
    	try{	
    		Component analyzerComp = clifAnalyzerApp.getComponentByName("analyzer");
    		analyzer = (AnalyzerExternalAccess) analyzerComp.getFcInterface(AnalyzerExternalAccess.ANALYZER_EXTERNAL_ACCESS);
    		
    	}
    	catch (NoSuchInterfaceException ex)
    	{
    		System.err.println(ex);
    	}
    }
    
    /**
     * Constructor
     * @param rsmd
     * @param testName
     * @param bladeId
     * @param eventTypeLabel 
     */
    ResultSet(IResultSetMetaData rsmd,String testName,String bladeId,String eventTypeLabel )
    {
    	this(rsmd);
    	try{
    		this.testName = testName;
    		this.bladeId = bladeId;
    		this.eventTypeLabel = eventTypeLabel;
    	}catch(Exception e){
    		this.result=null;
    	}
    }

    
    //////////////////////////
	// interface IResultset //
	//////////////////////////
    
    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getMetaData()
     */
    public IResultSetMetaData getMetaData() throws OdaException
    {
        return this.resultSetMetaData;
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#close()
     */
    public void close() throws OdaException
    {
        analyzer.closeEventIterator(iteratorKey);
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#setMaxRows(int)
     */
    public void setMaxRows( int max ) throws OdaException
    {
        this.maxRows = max;
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#next()
     */
    public boolean next( ) throws OdaException
	{	
    	try{
    		// first time to call next
    		if ( cursor == CURSOR_INITIAL_VALUE )
    		{
    			this.iteratorKey = analyzer.getEventIterator(testName, bladeId, eventTypeLabel, filter);
    			this.maxRows = analyzer.countEvents(this.testName, this.bladeId, this.eventTypeLabel, filter);
    		}   		
    		if ( ( this.maxRows <= 0 ? false : fetchAccumulator >= this.maxRows ) )
    		{
    			cursor = CURSOR_INITIAL_VALUE;
    			fetchAccumulator = 0; 			
    			return false;
    		}   		
    		if ( cursor == maxRows-1 )
    		{ 			
    			this.iteratorKey = analyzer.getEventIterator(testName, bladeId, eventTypeLabel, filter);
        		cursor = CURSOR_INITIAL_VALUE;
    			
    			if ( maxRows == 0 )
    			{
    				fetchAccumulator = 0;
    				return false;
    			}			
    		}
            
            // take the next event
    		BladeEvent[] bladesEvents = analyzer.getNextEvents(iteratorKey, 1);
    		if(bladesEvents != null && bladesEvents.length != 0){
    			result = bladesEvents[0];
    			fetchAccumulator++;
    			cursor++;
    		} else {
    			return false;
    		}
    		
    		return true;
    		
    	}catch(Exception e){
    		System.err.println("ResultSet.next():"+e);
    		return false;
    	}
    	
	}

    /* (non-Javadoc)
	 * @see org.eclipse.datatools.connectivity.oda.IResultSet#getRow()
	 */
    public int getRow() throws OdaException
    {
    	validateCursorState();
        return this.fetchAccumulator;
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getString(int)
     */
    public String getString( int index ) throws OdaException
    {
        validateCursorState();
        String originalColumnName = findColumnName(index);
        return result.getFieldValue(originalColumnName).toString();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getString(java.lang.String)
     */
    public String getString( String columnName ) throws OdaException
    {
        validateCursorState();
        String originalColumnName = findOriginalColumnName(columnName);
        return result.getFieldValue(originalColumnName).toString();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getInt(int)
     */
    public int getInt( int index ) throws OdaException
    {
        return stringToInt( getString( index ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getInt(java.lang.String)
     */
    public int getInt( String columnName ) throws OdaException
    {
        return stringToInt( getString( columnName ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getDouble(int)
     */
    public double getDouble( int index ) throws OdaException
    {
        return stringToDouble( getString( index ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getDouble(java.lang.String)
     */
    public double getDouble( String columnName ) throws OdaException
    {
        return stringToDouble( getString( columnName ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getBigDecimal(int)
     */
    public BigDecimal getBigDecimal( int index ) throws OdaException
    {
        return stringToBigDecimal( getString( index ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getBigDecimal(java.lang.String)
     */
    public BigDecimal getBigDecimal( String columnName ) throws OdaException
    {
        return stringToBigDecimal( getString( columnName ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getDate(int)
     */
    public Date getDate( int index ) throws OdaException
    {
        return stringToDate( getString( index ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getDate(java.lang.String)
     */
    public Date getDate( String columnName ) throws OdaException
    {
        return stringToDate( getString( columnName ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getTime(int)
     */
    public Time getTime( int index ) throws OdaException
    {
        return stringToTime( getString( index ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getTime(java.lang.String)
     */
    public Time getTime( String columnName ) throws OdaException
    {
        return stringToTime( getString( columnName ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getTimestamp(int)
     */
    public Timestamp getTimestamp( int index ) throws OdaException
    {
        return stringToTimestamp( getString( index ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getTimestamp(java.lang.String)
     */
    public Timestamp getTimestamp( String columnName ) throws OdaException
    {
        return stringToTimestamp( getString( columnName ) );
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getBlob(int)
     */
    public IBlob getBlob( int index ) throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getBlob(java.lang.String)
     */
    public IBlob getBlob( String columnName ) throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getClob(int)
     */
    public IClob getClob( int index ) throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getClob(java.lang.String)
     */
    public IClob getClob( String columnName ) throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getBoolean(int)
     */
    public boolean getBoolean( int index ) throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#getBoolean(java.lang.String)
     */
    public boolean getBoolean( String columnName ) throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#wasNull()
     */
    public boolean wasNull() throws OdaException
    {
        return this.wasNull;
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IResultSet#findColumn(java.lang.String)
     */
    public int findColumn( String columnName ) throws OdaException
    {
    	int index;
    	for(index=1;index<=this.resultSetMetaData.getColumnCount();index++)
    	{
    		if(this.resultSetMetaData.getColumnLabel(index).equalsIgnoreCase(columnName))
    		{
    			return index;
    		}
    		if(this.resultSetMetaData.getColumnName(index).equalsIgnoreCase(columnName))
    		{
    			return index;
    		}
    	}
    	return -1;
    }

    
    /////////////////////
	// private methods //
	/////////////////////
    
    /**
     * Validate whether the cursor has been initialized and at a valid row.
     * @throws OdaException if the cursor is not initialized
     */
    private void validateCursorState() throws OdaException
    {
    	if( this.cursor < 0 )
            throw new OdaException( Messages
                    .getString( "resultSet_CURSOR_HAS_NOT_BEEN_INITIALIZED" ) );
    }

    /**
     * Transform a String value to an int value
     * @param stringValue String value
     * @return Corresponding int value
     */
    private int stringToInt( String stringValue )
    {
    	if( stringValue != null )
        {
            try
            {
                return new Integer( stringValue ).intValue();
            }
            catch( NumberFormatException e )
            {
                this.wasNull = true;
            }
        }
        return 0;
    }

    /**
     * Transform a String value to a double value
     * @param stringValue String value
     * @return Corresponding double value
     */
    private double stringToDouble( String stringValue )
    {
    	if( stringValue != null )
        {
            try
            {
                return new Double( stringValue ).doubleValue();
            }
            catch( NumberFormatException e )
            {
                this.wasNull = true;
            }
        }
        return 0;
    }

    /**
     * Transform a String value to a big decimal value
     * @param stringValue String value
     * @return Corresponding BigDecimal value
     */
    private BigDecimal stringToBigDecimal( String stringValue )
    {
    	if( stringValue != null )
        {
            try
            {
                return new BigDecimal( stringValue );
            }
            catch( NumberFormatException e )
            {
                this.wasNull = true;
            }
        }
        return null;
    }

    /**
     * Transform a String value to a date value
     * @param stringValue String value
     * @return Corresponding date value
     * @throws OdaException 
     */

    private Date stringToDate( String stringValue ) throws OdaException
    {
    	if ( stringValue != null )
		{
			try
			{
				java.util.Date date = DateUtil.toDate( stringValue );
				return new Date( date.getTime( ) );
			}
			catch ( OdaException oe )
			{
			}
		}
		
		this.wasNull = true;
        return null;
    }

    /**
     * Transform a String value to a Time value
     * @param stringValue String value
     * @return Corresponding Time value
     * @throws OdaException 
     */
    private Time stringToTime( String stringValue ) throws OdaException
    {
    	if ( stringValue != null )
		{
			try
			{
				java.util.Date date = DateUtil.toDate( stringValue );
				return new Time( date.getTime( ) );
			}
			catch ( OdaException oe )
			{
			}
		}
		this.wasNull = true;
		return null;
    }

    /**
     * Transform a String value to a Timestamp value
     * @param stringValue String value
     * @return Corresponding Timestamp value
     */
    private Timestamp stringToTimestamp( String stringValue )
    {
    	if( stringValue != null )
        {
            try
            {
            	stringValue = stringValue.replaceAll("\\QT\\E"," ").split("\\QZ\\E")[0];
            	return Timestamp.valueOf( stringValue );
            }
            catch( IllegalArgumentException e )
            {
            	try{
            		long timeMills = new Long(stringValue).longValue();
            		return new Timestamp( timeMills );
            	}catch ( NumberFormatException e1)
            	{
            		try
					{
						java.util.Date date = DateUtil.toDate( stringValue );
						Timestamp timeStamp = new Timestamp( date.getTime( ) );

						return timeStamp;
					}
					catch ( OdaException oe )
					{
					}
            	}
            	
            }
        }
    	this.wasNull = true;
        return null;
    }
    
    /**
     * find the name of the column at the index
     * @param index
     * @return the name of the column
     */
    private String findColumnName(int index) throws OdaException
    {
    	return this.resultSetMetaData.getColumnName(index);
    }
    
    /**
     * find the original name of the column with the label columnLabel
     * @param columnLabel
     * @return the original name of the column
     */
    private String findOriginalColumnName(String columnLabel) throws OdaException
    {
    	String originalColumnName = null;
    	for(int i=1;i<=this.resultSetMetaData.getColumnCount();i++)
    	{
    		if(this.resultSetMetaData.getColumnLabel(i).equalsIgnoreCase(columnLabel))
    		{
    			originalColumnName = this.resultSetMetaData.getColumnName(i);
    			break;
    		}
    	}
    	return originalColumnName;
    }
  
}
