/*
 * CLIF is a Load Injection Framework
 * Copyright 2005 Manuel Azema, Tsirimiaina Andrianavonimiarina Jaona
 * Copyright 2006, 2009, 2010 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */

package org.ow2.clif.console.lib.egui;

import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.ow2.clif.deploy.ClifAppFacade;
import org.ow2.clif.deploy.ClifRegistry;
import org.ow2.clif.server.lib.ClifServerImpl;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.util.ExecutionContext;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;

/**
 * The main plugin class to be used in the desktop.
 *
 * @author Manuel AZEMA
 * @author Bruno Dillenseger
 * @author Florian Francheteau
 */
public class ClifConsolePlugin extends AbstractUIPlugin
{
	/**
	 * The CLIF properties file name
	 */
	static public final String CLIF_PROPS = "clif.props";
	/**
	 * Fractal ADL definition file of the CLIF application to deploy
	 */
	static public final String CLIF_APPLICATION = "org.ow2.clif.console.lib.egui.ClifApp";

	/**
	 * The shared instance.
	 */
	private static ClifConsolePlugin plugin;

	/**
	 * Currently connected registry
	 */
	private static ClifRegistry registry = null;

	/**
	 * Resource bundle.
	 */
	private ResourceBundle resourceBundle;

	/**
	 * The plugin id.
	 */
	public static final String PLUGIN_ID = "org.ow2.clif.console.plugin";
	
	private ClifConsole consoleView;
	
	private ClifAppFacade clifApp = null;

	/**
	 * On first call, creates the registry or connects to an existing registry,
	 * and creates the default local CLIF server. On subsequent calls, does nothing.
	 *
	 * @return the CLIF registry
	 * @throws ClifException if no registry could be created nor connected to, or
	 *                       if the default CLIF server could not be created.
	 */
	public static ClifRegistry getRegistry(boolean update)
		throws ClifException
	{
		if (registry == null || update)
		{
			try
			{
				registry = new ClifRegistry(false);
				System.out.println( "Connected to " + registry);
			}
			catch (Exception ex)
			{
				try
				{
					System.out.print("Creating a CLIF Registry... ");
					registry = new ClifRegistry(true);
					System.out.println(registry.toString());
				}
				catch (Throwable th)
				{
					throw new ClifException("Can't create nor connect to a CLIF registry: ", th);
				}
			}
			if (registry.lookupServer(ExecutionContext.DEFAULT_SERVER) == null)
			{
				ClifServerImpl.create(ExecutionContext.DEFAULT_SERVER, registry);
			}
		}
		return registry;
	}


	public ClifConsolePlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle(
					"org.ow2.clif.console.lib.egui.ClifConsolePluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		ExecutionContext.init(FileLocator.resolve(FileLocator.find(getBundle(), Path.EMPTY, null)).getPath());
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		ExecutionContext.WORKSPACE_PATH = root.getLocation().toOSString();
		File clifPropsFile = new File(ResourcesPlugin.getPlugin().getStateLocation().append(CLIF_PROPS).toOSString());
		if (!clifPropsFile.exists()) {
			clifPropsFile = new File(ExecutionContext.getBaseDir() + File.separator + ExecutionContext.PROPS_PATH);
		}
		try {
			ExecutionContext.setProperties(
					new FileInputStream(
							clifPropsFile));
			// set properties whose the processing is specific to Eclipse console
			setSpecialProperties(new FileInputStream(
					clifPropsFile));
		}
		catch (Exception ex) {
			throw new Exception(
					"Can't set system properties from file " + clifPropsFile.getName(),
					ex);
		}
		consoleView = new ClifConsole();
		ConsolePlugin.getDefault().getConsoleManager().addConsoles(
			new IConsole[] { consoleView } );
		clifApp = new ClifAppFacade(CLIF_APPLICATION, CLIF_APPLICATION);
	}

	/**
	 * set properties whose the processing is specific to Eclipse console
	 *
	 * @param ins InputStream which store CLIF properties
	 */
	private void setSpecialProperties(InputStream ins) {
		try {
			Properties props = new Properties();
			props.load(ins);
			String allParam = (String) props.get("clif.parameters");
			String[] parameters = allParam.split(" ");
			for (int i = 0; i < parameters.length; i++) {
				if (parameters[i].startsWith("-Dclif.codeserver.path")) {
					String[] property = parameters[i].split("=");
					String prop = completeProperty(property[1]);
					System.setProperty("clif.codeserver.path", prop);
				}
			}
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Scans all paths separated with ";" and modifies relative paths to
	 * absolute paths
	 *
	 * @param prop the property representing a list of paths
	 */
	private String completeProperty(String prop) {
		String completeProp = "";
		prop = prop.replaceAll("\"", "");
		String[] property = prop.split(";");
		for (int i = 0; i < property.length; i++) {
			Path pathFile = new Path(property[i]);
			if (pathFile.isAbsolute()) {
				completeProp += property[i];
			} else {
				IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
				pathFile = new Path(root.getLocation() + File.separator + property[i]);
				completeProp += pathFile.toOSString();
			}
			if (i < property.length - 1) {
				completeProp += ";";
			}
		}
		return completeProp;
	}


	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		consoleView.close();
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 *
	 * @return ClifConsolePlugin
	 */
	public static ClifConsolePlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 *
	 * @param key
	 * @return String
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = ClifConsolePlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 *
	 * @return ResourceBundle
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	public ClifAppFacade getClifApp()
	{
		return clifApp;
	}
}
